/* ____________________________________________________________________________

   ADOBE CONFIDENTIAL
   ___________________

    (c) Copyright 2009 Adobe Systems Incorporated
    All Rights Reserved.

   NOTICE:  All information contained herein is, and remains
   the property of Adobe Systems Incorporated and its suppliers,
   if any.  The intellectual and technical concepts contained
   herein are proprietary to Adobe Systems Incorporated and its
   suppliers and may be covered by U.S. and Foreign Patents,
   patents in process, and are protected by trade secret or copyright law.
   Dissemination of this information or reproduction of this material
   is strictly forbidden unless prior written permission is obtained
   from Adobe Systems Incorporated.
   ____________________________________________________________________________
*/

#include "stdafx.h"

#include <string.h>
#include <stdio.h>
#include <tchar.h>

#include "npfunctions.h"

static bool sDidRequest = false;

/*	Logging macros	*/
#if XP_MACOSX
	#define _T(x)			x
#endif

#if _DEBUG

	#if XP_MACOSX
		#define PLUGIN_LOG_STR(logStr) printf( "%s\n", logStr )
	#else
		#define PLUGIN_LOG_STR(logStr)	OutputDebugString(logStr); OutputDebugString("\r\n")
	#endif

	#define PLUGIN_LOG(...)		do { TCHAR logStr[1000]; _stprintf_s( logStr, 1000, __VA_ARGS__ ); PLUGIN_LOG_STR( logStr ); } while (0)

#else
	#define PLUGIN_LOG(...)
#endif

#if defined(XP_MACOSX)
	#define PLUGIN_LOG(...)		do { printf( _VA_ARGS__; printf("\n"); } while (0)
#else
#endif

extern NPNetscapeFuncs gBrowserFuncTable;

/*	---------------------------------------------------------- */

NPError OSCALL NP_Initialize( NPNetscapeFuncs *browserFuncs )
{
	PLUGIN_LOG( _T("NP_Initialize") );
	
	size_t sizeToCopy = browserFuncs->size;
	if ( sizeToCopy > sizeof( gBrowserFuncTable ) )
		sizeToCopy = sizeof( gBrowserFuncTable );
	memmove( (void *) &gBrowserFuncTable, (void *) browserFuncs, sizeToCopy );
	gBrowserFuncTable.size = (uint16_t) sizeToCopy;

    return NPERR_NO_ERROR;
}

NPError OSCALL NP_GetEntryPoints( NPPluginFuncs *pluginFuncs )
{
	PLUGIN_LOG( "NP_GetEntryPoints" );

    pluginFuncs->version = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
    pluginFuncs->size = sizeof( pluginFuncs );

    pluginFuncs->newp = NPP_New;
    pluginFuncs->destroy = NPP_Destroy;
    pluginFuncs->setwindow = NPP_SetWindow;
    pluginFuncs->newstream = NPP_NewStream;
    pluginFuncs->destroystream = NPP_DestroyStream;
    pluginFuncs->asfile = NPP_StreamAsFile;
    pluginFuncs->writeready = NPP_WriteReady;
    pluginFuncs->write = NPP_Write;
    pluginFuncs->print = NPP_Print;
    pluginFuncs->event = NPP_HandleEvent;
    pluginFuncs->urlnotify = NPP_URLNotify;
    pluginFuncs->getvalue = NPP_GetValue;
    pluginFuncs->setvalue = NPP_SetValue;
    
    return NPERR_NO_ERROR;
}

char *NP_GetPluginVersion()
{
	PLUGIN_LOG( ("NP_GetPluginVersion") );

	return "TestPlugin 1.0";
}

char *NP_GetMIMEDescription()
{
	PLUGIN_LOG( ("NP_MIMEDescription") );

	/*	Return a list separated by semicolons.
		Each item in the list is three things separated by colons:
		"mimetype:extension:description" */
	return "application/pdf:pdf:Test Plugin Document";
}

NPError OSCALL NP_Shutdown( void )
{
	PLUGIN_LOG( ("NP_Shutdown") );

	return NPERR_NO_ERROR;
}

/*	---------------------------------------------------------- */

NPError NPP_New( NPMIMEType pluginType, NPP instance, uint16_t mode, int16_t argc, char *argn[], char *argv[], NPSavedData *saved )
{
	PLUGIN_LOG( ("NPP_New") );

    return NPERR_NO_ERROR;
}

NPError NPP_Destroy( NPP instance, NPSavedData **save )
{
	PLUGIN_LOG( ("NPP_Destroy") );

    return NPERR_NO_ERROR;
}

NPError NPP_SetWindow( NPP instance, NPWindow *window )
{
	PLUGIN_LOG( ("NPP_SetWindow") );

    return NPERR_NO_ERROR;
}
 

NPError NPP_NewStream( NPP instance, NPMIMEType type, NPStream *stream, NPBool seekable, uint16_t *stype )
{
	PLUGIN_LOG( ("NPP_NewStream") );

    *stype = NP_SEEK;
    return NPERR_NO_ERROR;
}

NPError NPP_DestroyStream( NPP instance, NPStream *stream, NPReason reason )
{
	PLUGIN_LOG( ("NPP_DestroyStream") );

    return NPERR_NO_ERROR;
}

int32_t NPP_WriteReady( NPP instance, NPStream *stream )
{
	PLUGIN_LOG( ("NPP_WriteReady") );

    return 1024;
}

int32_t NPP_Write( NPP instance, NPStream *stream, int32_t offset, int32_t len, void *buffer )
{
	PLUGIN_LOG( ("NPP_Write") );

	if ( !sDidRequest )
	{
		uint32_t end = stream->end;

		sDidRequest = true;

		NPByteRange br1;
		NPByteRange br2;

		br1.offset = 1;
		br1.length = 1;
		br1.next = &br2;

		br2.offset = end - 2000;
		br2.length = 2000;
		br2.next = NULL;

		NPN_RequestRead( stream, &br1 );
	}

    return len;
}

void NPP_StreamAsFile( NPP instance, NPStream *stream, const char *fname )
{
	PLUGIN_LOG( ("NPP_StreamAsFile") );
}

void NPP_Print( NPP instance, NPPrint *platformPrint )
{
	PLUGIN_LOG( ("NPP_Print") );
}

int16_t NPP_HandleEvent( NPP instance, void *event )
{
	#if defined(XP_MACOSX)
		/*	null events come very quickly, don't log them. */
		if ( ((EventRecord *) event)->what != nullEvent )
		{
			PLUGIN_LOG( ("NPP_HandleEvent") );
		}
	#endif

    return 0;
}

void NPP_URLNotify( NPP instance, const char *url, NPReason reason, void *notifyData )
{
	PLUGIN_LOG( ("NPP_URLNotify") );
}

NPError NPP_GetValue( NPP instance, NPPVariable variable, void *value )
{
	PLUGIN_LOG( ("NPP_GetValue") );

    return NPERR_GENERIC_ERROR;
}

NPError NPP_SetValue( NPP instance, NPNVariable variable, void *value )
{
	PLUGIN_LOG( ("NPP_SetValue") );

    return NPERR_GENERIC_ERROR;
}

/*	---------------------------------------------------------- */




